import os

import streamlit as st


def home_page():
    st.subheader("🏠 Codeas")

    # Add API key inputs
    st.sidebar.subheader("API Keys")
    openai_api_key = st.sidebar.text_input(
        "OpenAI API Key", value=os.environ.get("OPENAI_API_KEY", ""), type="password"
    )
    anthropic_api_key = st.sidebar.text_input(
        "Anthropic API Key",
        value=os.environ.get("ANTHROPIC_API_KEY", ""),
        type="password",
    )
    google_api_key = st.sidebar.text_input(
        "Google API Key", value=os.environ.get("GOOGLE_API_KEY", ""), type="password"
    )

    # Set environment variables if API keys are provided
    if openai_api_key:
        os.environ["OPENAI_API_KEY"] = openai_api_key
    if anthropic_api_key:
        os.environ["ANTHROPIC_API_KEY"] = anthropic_api_key
    if google_api_key:
        os.environ["GOOGLE_API_KEY"] = google_api_key

    st.markdown(
        """
    Codeas is a tool that helps you **boost your software development processes using generative AI**.
    """
    )
    st.markdown(
        """
    The following use cases are currently implemented:
    """
    )
    st.page_link("pages/1_📚_Documentation.py", label="Documentation", icon="📚")
    st.page_link("pages/2_🚀_Deployment.py", label="Deployment", icon="🚀")
    st.page_link("pages/3_🧪_Testing.py", label="Testing", icon="🧪")
    st.page_link("pages/4_🔄_Refactoring.py", label="Refactoring", icon="🔄")

    st.markdown(
        """
    Some additional features include: \n
    **A chat interface with access to the entire codebase:**
    """
    )
    st.page_link("pages/5_💬_Chat.py", label="Chat", icon="💬")

    st.markdown(
        """
    **A prompt editor to generate, save and modify the prompts used in the chat interface:**
    """
    )
    st.page_link("pages/6_📝_Prompts.py", label="Prompts", icon="📝")

    st.markdown(
        """
    **A usage tracker to monitor the number of tokens and cost generated by the tool:**
    """
    )
    st.page_link("pages/7_🔍_Usage.py", label="Usage", icon="🔍")
    st.markdown(
        """
    For more information about the tool, visit the [GitHub repository](https://github.com/DivergerThinking/codeas).
    """
    )


if __name__ == "__main__":
    home_page()
